********************************************************************************
// Purpose: Pull in FSA data and clean it
// Author: Jared Colston
// Updated for 2012-2022 by: Jacob Martin
********************************************************************************

clear all

********************************************************************************
**# [1] FILE PATHWAYS ---------------------------------------------------------|
	
	global path "C:\Users\marti\PNPI Dropbox\Jacob Martin\Jacob\Data\fsa"
	
	capture mkdir "$path\raw"
	capture mkdir "$path\temp"
	capture mkdir "$path\backup"
	
	global raw "$path\raw"
	global temp "$path\temp"
	global backup "$path\backup"
	global analysis "$path"

********************************************************************************
**# [2] GET DATA FROM FSA DATA CENTER -----------------------------------------|

// Special handling for 2011
local yr = 2011
copy https://studentaid.gov/sites/default/files/DL_Dashboard_AY2011_2012_Q4.xls "$raw\dl_2011.xlsx", replace
import excel "$raw\dl_2011.xlsx", sheet("Award Year Summary") cellrange(A7) clear
rename (A B C D E F G H I J K L M N O P Q R S T U V W X Y Z AA AB ///
        AC AD AE AF AG AH AI) ///
       (opeid instnm stabbr zipcode sch_type ug_sub_recip ug_sub_orig_num ///
        ug_sub_orig_amt ug_sub_disb_num ug_sub_disb_amt gr_sub_recip ///
        gr_sub_orig_num gr_sub_org_amt gr_sub_disb_num gr_sub_orig_amt ///
        ug_unsub_recip ug_unsub_orig_num ug_unsub_orig_amt ///
        ug_unsub_disb_num ug_unsub_disb_amt gr_unsub_recip ///
        gr_unsub_orig_num gr_unsub_org_amt gr_unsub_disb_num ///
        gr_unsub_orig_amt pplus_recip pplus_orig_num pplus_orig_amt ///
        pplus_disb_num pplus_disb_amt gplus_recip gplus_orig_num ///
        gplus_orig_amt gplus_disb_num gplus_disb_amt)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2011"
save "$temp\dl_2011.dta", replace

// Loop for 2012-2022
forvalues x = 2012/2022 {
    local yr = `x' + 1
    if inlist(`x',2012,2013) {
        local filepath "https://studentaid.gov/sites/default/files/"
        local cell A7
        local filename1 DL_
        local filename2 AY
        local filename3 Q
        local filename4 Dashboard_
        local yr2 _
        local yr3 _
    }
    if inlist(`x',2014,2015,2016) {
        local filepath "https://studentaid.gov/sites/default/files/fsawg/datacenter/library/"
        local cell A7
        local filename1 DL_
        local filename2 AY
        local filename3 Q
        local filename4 Dashboard_
        local yr2 _
        local yr3 _
    }
    if inlist(`x',2017) {
        local filepath "https://studentaid.gov/sites/default/files/fsawg/datacenter/library/"
        local cell A8
        local filename1 DL_
        local filename2 AY
        local filename3 Q
        local filename4 Dashboard_
        local yr2 _
        local yr3 _
    }
    if inrange(`x',2018,2022) {
        local filepath "https://studentaid.gov/sites/default/files/fsawg/datacenter/library/"
        local cell A8
        local filename1 dl-
        local filename2 ay
        local filename3 q
        local filename4 dashboard-
        local yr2 -
        local yr3 -
    }		
    copy "`filepath'`filename1'`filename4'`filename2'`x'`yr2'`yr'`yr3'`filename3'4.xls" "$raw\dl_`x'.xlsx", replace
    import excel "$raw\dl_`x'.xlsx", sheet("Award Year Summary") cellrange(`cell') clear
    rename 	(A B C D E F G H I J K L M N O P Q R S T U V W X Y Z AA AB ///
                AC AD) ///
            (opeid instnm stabbr zipcode sch_type ug_sub_recip ug_sub_orig_num ///
            ug_sub_orig_amt ug_sub_disb_num ug_sub_disb_amt ///
            ug_unsub_recip ug_unsub_orig_num ug_unsub_orig_amt ///
            ug_unsub_disb_num ug_unsub_disb_amt gr_unsub_recip ///
            gr_unsub_orig_num gr_unsub_org_amt gr_unsub_disb_num ///
            gr_unsub_orig_amt pplus_recip pplus_orig_num pplus_orig_amt ///
            pplus_disb_num pplus_disb_amt gplus_recip gplus_orig_num ///
            gplus_orig_amt gplus_disb_num gplus_disb_amt)
    
    capture replace zipcode = subinstr(zipcode,"-","",.)

    ds, has(type string)
    foreach y in `r(varlist)' {
        replace `y' = trim(`y')
        replace `y' = "" if `y' == "-"
        destring `y', replace
    }
    
    drop if opeid == .
    gen year = "`x'"
    save "$temp\dl_`x'.dta", replace
}

// Append the data files
use "$temp\dl_2011.dta", clear
forvalues i = 2012/2022 {
    append using "$temp\dl_`i'.dta"
}
drop AE AF zipcode
save "$temp\dl_comb.dta", replace

********************************************************************************

// Grant volume 

// Loop for 2011-2018
foreach x in 1112 1213 1314 1415 1516 1617 1718 {
    local yr = "20" + substr("`x'", 1, 2)
    if inlist(`x', 1718) local cell A8
    else local cell A7
    if inlist(`x', 1112, 1213) local sheet "Q4 `x' YTD"
    else local sheet "Award Year Summary"
    
    if inlist(`x', 1112, 1213) {
        local filepath https://studentaid.gov/sites/default/files/
    }
    if inlist(`x', 1314, 1415, 1516, 1617, 1718) {
        local filepath https://studentaid.gov/sites/default/files/fsawg/datacenter/library/
    }
    
    copy "`filepath'Q4`x'AY.xls" "$raw\grant_`yr'.xlsx", replace
    import excel "$raw\grant_`yr'.xlsx", sheet(`sheet') cellrange(`cell') clear
    rename (A B C D E F G H I J K) ///
        (opeid instnm stabbr zipcode sch_type pell_recip pell_amt teach_recip ///
         teach_amt iraq_recip iraq_amt)
    
    ds, has(type string)
    foreach y in `r(varlist)' {
        replace `y' = trim(`y')
        replace `y' = "" if `y' == "-"
        destring `y', replace
    }
    
    drop if opeid == .
    gen year = "`yr'"
    save "$temp\grant_`yr'.dta", replace		
}

// Extend loop for 2018-2022
foreach x in 18 19 20 21 22 {
    local x2 = `x' + 1
    local yr = "20" + "`x'"
    copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/grants-ay`x'-`x2'-q4.xls "$raw\grant_`yr'.xlsx", replace
    import excel "$raw\grant_`yr'.xlsx", sheet("Award Year Summary") cellrange(A8) clear
    rename (A B C D E F G H I J K) ///
        (opeid instnm stabbr zipcode sch_type pell_recip pell_amt teach_recip ///
         teach_amt iraq_recip iraq_amt)
    
    ds, has(type string)
    foreach y in `r(varlist)' {
        replace `y' = trim(`y')
        replace `y' = "" if `y' == "-"
        destring `y', replace
    }
    
    drop if opeid == .
    gen year = "`yr'"
    save "$temp\grant_`yr'.dta", replace		
}

// Append
use "$temp\grant_2011.dta", clear 
forvalues i = 2012/2022 {
    append using "$temp\grant_`i'.dta"
}

drop L zipcode
save "$temp\grant_comb.dta", replace

********************************************************************************

// Campus-based programs 
// Formatting is completely erratic, not much looping possible

// 2011
copy https://studentaid.gov/sites/default/files/2011-2012CBData.xls "$raw\campusbased_2011.xlsx", replace
import excel "$raw\campusbased_2011.xlsx", sheet("AY2011-12") cellrange(A5) clear
rename (A B C D E F G H I J K L M N) ///
    (opeid instnm stabbr zipcode sch_type fws_recip fws_award fws_disb ///
     perkins_recip perkins_award perkins_disb seog_recip seog_award ///
     seog_disb)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2011"
save "$temp\campusbased_2011.dta", replace

// 2012
copy https://studentaid.gov/sites/default/files/2012-13CampusBasedFunding.xls "$raw\campusbased_2012.xlsx", replace
import excel "$raw\campusbased_2012.xlsx", sheet("2012-13") cellrange(A5) clear
rename (A B C D E F G H I J K L M N) ///
    (opeid instnm stabbr zipcode sch_type fws_recip fws_award fws_disb ///
     perkins_recip perkins_award perkins_disb seog_recip seog_award ///
     seog_disb)

replace zipcode = subinstr(zipcode,"-","",.)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2012"
save "$temp\campusbased_2012.dta", replace

// 2013
copy https://studentaid.gov/sites/default/files/2013-14CampusBased.xls "$raw\campusbased_2013.xlsx", replace
import excel "$raw\campusbased_2013.xlsx", sheet("AY2013-14") cellrange(A5) clear
rename (A B C D E F G H I J K L M N) ///
    (opeid instnm stabbr zipcode sch_type fws_recip fws_award fws_disb ///
     perkins_recip perkins_award perkins_disb seog_recip seog_award ///
     seog_disb)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2013"
save "$temp\campusbased_2013.dta", replace

// 2014
copy https://studentaid.gov/sites/default/files/2014-15CampusBased.xls "$raw\campusbased_2014.xlsx", replace
import excel "$raw\campusbased_2014.xlsx", sheet("AY2014-15") cellrange(A5) clear
rename (A B C D E F G H I J K L M N) ///
    (opeid instnm stabbr zipcode sch_type fws_recip fws_award fws_disb ///
     perkins_recip perkins_award perkins_disb seog_recip seog_award ///
     seog_disb)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2014"
save "$temp\campusbased_2014.dta", replace

// 2015
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/2015-16CampusBased.xls "$raw\campusbased_2015.xlsx", replace
import excel "$raw\campusbased_2015.xlsx", sheet("2015-2016 CB Data") cellrange(A5) clear
rename (A B C D E F G H I J K L M N) ///
    (opeid instnm stabbr zipcode sch_type fws_recip fws_award fws_disb ///
     perkins_recip perkins_award perkins_disb seog_recip seog_award ///
     seog_disb)

replace zipcode = subinstr(zipcode,"-","",.)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2015"
save "$temp\campusbased_2015.dta", replace

// 2016
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/2016-17CampusBased.xls "$raw\campusbased_2016.xlsx", replace
import excel "$raw\campusbased_2016.xlsx", sheet("2016-17 CB Data") cellrange(A5) clear
rename (A B C D E F G H I J K L M) ///
    (opeid instnm stabbr zipcode sch_type perkins_recip perkins_disb ///
     seog_award seog_recip seog_disb fws_award fws_recip fws_disb)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2016"
save "$temp\campusbased_2016.dta", replace

// 2017
copy https://studentaid.gov/sites/default/files/ay17-18-campus-based.xls "$raw\campusbased_2017.xlsx", replace
import excel "$raw\campusbased_2017.xlsx", sheet("2017-18 CB Data") cellrange(A5) clear
rename (A B C D E F G H I J K L) ///
    (opeid instnm stabbr sch_type perkins_recip perkins_disb ///
     seog_award seog_recip seog_disb fws_award fws_recip fws_disb)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2017"
save "$temp\campusbased_2017.dta", replace

// 2018
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/ay18-19-campus-based.xls "$raw\campusbased_2018.xlsx", replace
import excel "$raw\campusbased_2018.xlsx", sheet("2018-2019 CB Data") cellrange(A4) clear
rename (A B C D E F G H I J K L M) ///
    (opeid instnm stabbr zipcode sch_type perkins_recip perkins_disb ///
     seog_award seog_recip seog_disb fws_award fws_recip fws_disb)

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2018"
save "$temp\campusbased_2018.dta", replace

// 2019
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/ay19-20-campus-based.xls "$raw\campusbased_2019.xlsx", replace
import excel "$raw\campusbased_2019.xlsx", sheet("2019-2020") cellrange(A5) clear
rename (A B C D E F G H I J K) ///
    (opeid instnm stabbr zipcode sch_type seog_award seog_recip ///
     seog_disb fws_award fws_recip fws_disb)

replace opeid = "" if regexm(opeid,"[0-9]$") == 0 // Remove non-numeric strings

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2019"
save "$temp\campusbased_2019.dta", replace

// 2020
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/ay20-21-campus-based.xls "$raw\campusbased_2020.xlsx", replace
import excel "$raw\campusbased_2020.xlsx", sheet("2020-2021") cellrange(A5) clear
rename (A B C D E F G H I J K) ///
    (opeid instnm stabbr zipcode sch_type seog_award seog_recip ///
     seog_disb fws_award fws_recip fws_disb)

replace opeid = "" if regexm(opeid,"[0-9]$") == 0 // Remove non-numeric strings

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2020"
save "$temp\campusbased_2020.dta", replace

// 2021
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/ay21-22-campus-based.xls "$raw\campusbased_2021.xlsx", replace
import excel "$raw\campusbased_2021.xlsx", sheet("AY21-22") cellrange(A5) clear
rename (A B C D E F G H I J K) ///
    (opeid instnm stabbr zipcode sch_type seog_award seog_recip ///
     seog_disb fws_award fws_recip fws_disb)

replace opeid = "" if regexm(opeid,"[0-9]$") == 0 // Remove non-numeric strings

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2021"
save "$temp\campusbased_2021.dta", replace

// Address string variables
use "$temp\campusbased_2021.dta", clear

// Inspect non-numeric values
list seog_recip if missing(real(seog_recip)) | seog_recip == ""
list seog_disb if missing(real(seog_disb)) | seog_disb == ""
list fws_recip if missing(real(fws_recip)) | fws_recip == ""
list fws_disb if missing(real(fws_disb)) | fws_disb == ""

// Convert specific problematic values to missing
replace seog_recip = "" if seog_recip == "<10"  // Clean non-numeric characters like "<10"
replace seog_disb = "" if seog_disb == "<10"
replace fws_recip = "" if fws_recip == "<10"
replace fws_disb = "" if fws_disb == "<10"

// Convert to numeric and force any remaining non-numeric to missing
destring seog_recip, replace force
destring seog_disb, replace force
destring fws_recip, replace force
destring fws_disb, replace force

// Ensure all specified variables are long
recast long seog_recip seog_disb fws_recip fws_disb

// Save the cleaned dataset
save "$temp\campusbased_2021.dta", replace

// 2022
copy https://studentaid.gov/sites/default/files/fsawg/datacenter/library/ay22-23-campus-based.xls "$raw\campusbased_2022.xlsx", replace
import excel "$raw\campusbased_2022.xlsx", sheet("AY22-23") cellrange(A5) clear
rename (A B C D E F G H I J K) ///
    (opeid instnm stabbr zipcode sch_type seog_award seog_recip ///
     seog_disb fws_award fws_recip fws_disb)

replace opeid = "" if regexm(opeid,"[0-9]$") == 0 // Remove non-numeric strings

ds, has(type string)
foreach y in `r(varlist)' {
    replace `y' = trim(`y')
    replace `y' = "" if `y' == "-"
    destring `y', replace
}

drop if opeid == .
gen year = "2022"
save "$temp\campusbased_2022.dta", replace

use "$temp\campusbased_2022.dta", clear

// Inspect non-numeric values
list seog_recip if missing(real(seog_recip)) | seog_recip == ""
list seog_disb if missing(real(seog_disb)) | seog_disb == ""
list fws_recip if missing(real(fws_recip)) | fws_recip == ""
list fws_disb if missing(real(fws_disb)) | fws_disb == ""

// Convert specific problematic values to missing
replace seog_recip = "" if seog_recip == "<10"  // Clean non-numeric characters like "<10"
replace seog_disb = "" if seog_disb == "<10"
replace fws_recip = "" if fws_recip == "<10"
replace fws_disb = "" if fws_disb == "<10"

// Convert to numeric and force any remaining non-numeric to missing
destring seog_recip, replace force
destring seog_disb, replace force
destring fws_recip, replace force
destring fws_disb, replace force

// Ensure all specified variables are long
recast long seog_recip seog_disb fws_recip fws_disb

// Save the cleaned dataset
save "$temp\campusbased_2022.dta", replace

// Convert 2011-2019 datasets
forvalues year = 2011/2019 {
    use "$temp\campusbased_`year'.dta", clear
    recast long seog_recip seog_disb fws_recip fws_disb
    save "$temp\campusbased_`year'.dta", replace
}

// Append all individual year datasets into one combined dataset
use "$temp\campusbased_2011.dta", clear
forvalues year = 2012/2022 {
    append using "$temp\campusbased_`year'.dta"
}

// Drop any unnecessary variables or clean if needed
drop zipcode  // Optional: Drop 'zipcode' if not needed

// Save the combined dataset
save "$temp\campusbased_comb.dta", replace

********************************************************************************

**# [3] COMBINE & CLEAN DATA --------------------------------------------------|	 

// Load the combined loan data
use "$temp\dl_comb.dta", clear

// Merge with grant data
merge 1:1 opeid year using "$temp\grant_comb.dta"
drop _merge

// Merge with campus-based program data
merge 1:1 opeid year using "$temp\campusbased_comb.dta"
drop _merge 

// Clean and standardize the `sch_type` variable
replace sch_type = strproper(sch_type)  // Convert to proper case (e.g., "University" instead of "UNIVERSITY")
replace sch_type = subinstr(sch_type, "-", " ", .)  // Replace dashes with spaces

// Ensure `opeid` is a string and create a standardized version `opeid2`
tostring opeid, replace
gen opeid2 = substr("00000000", 1, 8 - length(opeid)) + opeid  // Create a zero-padded version of opeid with a length of 8

// Drop data from the year 2011 (not relevant for 2012-2022)
drop if year == "2011"

// Save the cleaned dataset
save "$analysis\fsa_cleaned.dta", replace

